from __future__ import annotations

from argparse import Namespace
from typing import TYPE_CHECKING, Any, Callable, Type

if TYPE_CHECKING:
    from os import PathLike


def import_pyfile(pyfile: PathLike | str) -> dict:
    """Import a python file and return the globals dictionary"""
    import importlib.util

    spec = importlib.util.spec_from_file_location("config", pyfile)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    try:
        return module.args
    except AttributeError:
        raise AttributeError("No `args` variables found") from None


def get_ns_dest(namespace: Namespace, dest: str) -> tuple[Namespace, str]:
    """Get the namespace and the last part of the dest to update"""
    if "." not in dest:
        return namespace, dest

    # Split the destination into a list of keys
    keys = dest.split(".")
    ns = namespace
    for key in keys[:-1]:
        value = getattr(ns, key, None)
        if value is None:
            value = Namespace()
            setattr(ns, key, value)
        ns = value

    return ns, keys[-1]


def copy_items(items: Any) -> Any:  # pragma: no cover
    """Copy items if needed, copied from argparse"""
    if items is None:
        return []
    # The copy module is used only in the 'append' and 'append_const'
    # actions, and it is needed only when the default value isn't a list.
    # Delay its import for speeding up the common case.
    if type(items) is list:
        return items[:]
    import copy
    return copy.copy(items)


def add_attribute(attr: str, default: Any = None) -> Callable[[Type], Type]:
    """Add an attribute to a class, working as a decorator"""
    def deco(cls):
        old_init = cls.__init__

        def new_init(self, *args, **kwargs):
            value = kwargs.pop(attr, default)
            setattr(self, attr, value)
            old_init(self, *args, **kwargs)

        cls.__init__ = new_init
        return cls

    return deco


def showable(obj: Any) -> bool:
    """Return True if the object is showable"""
    return getattr(obj, "show", True)



from __future__ import annotations

import sys
from typing import IO, TYPE_CHECKING, Any, Sequence
from pathlib import Path
from gettext import gettext as _
from argparse import (
    SUPPRESS,
    _SubParsersAction,
    _ArgumentGroup as APArgumentGroup,
    ArgumentParser as APArgumentParser,
    Namespace,
)

from simpleconf import Config

from . import type_
from .utils import (
    get_ns_dest,
    import_pyfile,
    add_attribute,
    showable,
)
from .action import (
    StoreAction,
    StoreConstAction,
    StoreTrueAction,
    StoreFalseAction,
    AppendAction,
    AppendConstAction,
    CountAction,
    ExtendAction,
    ListAction,
    HelpAction,
)
from .formatter import ChargedHelpFormatter

if TYPE_CHECKING:
    from argparse import _ActionT, _FormatterClass


@add_attribute("level", 0)
class ArgumentParser(APArgumentParser):
    """Class to handle command line arguments and configuration files"""

    def __init__(
        self,
        prog: str | None = None,
        usage: str | None = None,
        description: str | None = None,
        epilog: str | None = None,
        parents: Sequence[ArgumentParser] = [],
        formatter_class: _FormatterClass = ChargedHelpFormatter,
        prefix_chars: str = "-",
        fromfile_prefix_chars: str | None = None,
        argument_default: Any = None,
        conflict_handler: str = "error",
        add_help: bool | str = True,
        allow_abbrev: bool = True,
        exit_on_error: bool = True,
        exit_on_void: bool = False,
    ) -> None:
        old_add_help = add_help
        add_help = False
        kwargs = {
            "prog": prog,
            "usage": usage,
            "description": description,
            "epilog": epilog,
            "parents": parents,
            "formatter_class": formatter_class,
            "prefix_chars": prefix_chars,
            "fromfile_prefix_chars": fromfile_prefix_chars,
            "argument_default": argument_default,
            "conflict_handler": conflict_handler,
            "add_help": add_help,
            "allow_abbrev": allow_abbrev,
        }
        if sys.version_info >= (3, 9):  # pragma: no cover
            kwargs["exit_on_error"] = exit_on_error
        super().__init__(**kwargs)
        self.exit_on_void = exit_on_void

        # Register our actions to override argparse's or add new ones
        self.register("action", None, StoreAction)
        self.register("action", "store", StoreAction)
        self.register("action", "store_const", StoreConstAction)
        self.register("action", "store_true", StoreTrueAction)
        self.register("action", "store_false", StoreFalseAction)
        self.register("action", "append", AppendAction)
        self.register("action", "append_const", AppendConstAction)
        self.register("action", "count", CountAction)
        self.register("action", "extend", ExtendAction)
        self.register("action", "list", ListAction)
        self.register("action", "help", HelpAction)
        self.register("type", "py", type_.py)
        self.register("type", "json", type_.json)
        self.register("type", "path", Path)
        self.register("type", "auto", type_.auto)

        # Add help option to support + for more options
        default_prefix = (
            "-" if "-" in self.prefix_chars else self.prefix_chars[0]
        )
        if old_add_help is True:
            self.add_argument(
                f"{default_prefix}h",
                f"{default_prefix * 2}help",
                action="help",
                default=SUPPRESS,
                help=_("show help message and exit"),
            )
        elif old_add_help == "+":
            self.add_argument(
                f"{default_prefix}h",
                f"{default_prefix * 2}help",
                f"{default_prefix}h+",
                f"{default_prefix * 2}help+",
                action="help",
                default=SUPPRESS,
                help=_(
                    "show help message (with + to show more options) and exit"
                ),
            )
        # restore add_help
        self.add_help = old_add_help  # type: ignore[assignment]

    def add_subparser(self, name: str, **kwargs) -> ArgumentParser | Any:
        """Add a subparser directly.

        Instead of
        >>> subparsers = parser.add_subparsers(...)
        >>> parser_a = subparsers.add_parser("a", ...)
        >>> parser_b = subparsers.add_parser("b", ...)

        Now we can do
        >>> parser_a = parser.add_subparser("a", ...)

        Subparsers are automatically created when needed. This way, the
        title will be set to "subcommands" and the dest will be set to
        "COMMAND" or "COMMAND{level+1}" if the parser is not at level 0
        (main parser). The subparsers will be required.

        If you need to customize the subparsers, you can still use the
        add_subparsers method in standard argparse way.

        Args:
            name (str): The name of the subparser
            **kwargs: The arguments to pass to add_parser()

        Returns:
            ArgumentParser: The subparser
        """
        if self._subparsers is None:
            self._subparsers = self.add_subparsers(  # type: ignore[assignment]
                title=_("subcommands"),
                required=True,
                dest="COMMAND"
                if self.level == 0
                else f"COMMAND{self.level+1}",
            )
        kwargs.setdefault("help", f"The {name} command")
        return self._subparsers.add_parser(
            name, level=self.level + 1, **kwargs
        )

    add_command = add_subparser

    def parse_known_args(
        self,
        args: Sequence[str] | None = None,
        namespace: Namespace | None = None,
    ) -> tuple[Namespace, list[str]]:
        """Parse known arguments.

        Modify to handle exit_on_void and @file to load default values.
        """
        if args is None:  # pragma: no cover
            # args default to the system args
            args = sys.argv[1:]
        else:
            # make sure that args are mutable
            args = list(args)

        if not self.fromfile_prefix_chars:
            new_args = args
        else:
            # Setup the defaults if a configuration file is given by @config.ini
            new_args = []
            for arg in args:
                if (
                    not arg
                    or arg[0] not in self.fromfile_prefix_chars
                    or arg.endswith(".txt")
                ):
                    new_args.append(arg)
                else:
                    try:
                        conf = arg[1:]
                        if conf.endswith(".py"):
                            conf = import_pyfile(conf)
                        self.set_defaults_from_configs(conf)
                    except Exception as e:
                        self.error(f"Cannot import [{conf}]: {e}")

        # default Namespace built from parser defaults
        if namespace is None:
            namespace = Namespace()

        # add any action defaults that aren't present
        # Do this mainly for namespace actions, like "--group.abc"
        for action in self._actions:
            if "." not in action.dest:
                # Leave the normal actions to the super class
                continue

            ns, last_key = get_ns_dest(namespace, action.dest)
            if not hasattr(ns, last_key):
                setattr(ns, last_key, action.default)

        parsed_args, argv = super().parse_known_args(new_args, namespace)
        if not argv and not args and self.exit_on_void:
            self.error("No arguments provided")

        return parsed_args, argv

    def set_defaults_from_configs(
        self,
        *configs: dict | str,
        optionalize: bool = True,
    ) -> None:
        """Set default values from configs.

        Args:
            *configs (dict | str): The configs to load, either a dict or
                a configuration file.
            optionalize (bool): Whether to make the arguments optional if
                they are required.
        """
        conf = Config.load(*configs)
        for action in self._actions:
            if "." not in action.dest and action.dest in conf:
                action.default = conf[action.dest]
                if optionalize:
                    action.required = False
            elif "." in action.dest:
                parts = action.dest.split(".")
                try:
                    cf = conf
                    for part in parts[:-1]:
                        cf = cf[part]
                    action.default = cf[parts[-1]]
                    if optionalize:
                        action.required = False
                except KeyError:
                    continue
            # see if we need to update subparsers
            if isinstance(action, _SubParsersAction):
                for name, subparser in action._name_parser_map.items():
                    if name in conf:
                        subparser.set_defaults_from_configs(
                            conf[name],
                            optionalize=optionalize,
                        )

    def _add_action(self, action: _ActionT) -> _ActionT:
        """Add an action to the parser.

        Modify to handle namespace actions, like "--group.abc"
        """
        if "." not in action.dest or isinstance(action, APArgumentGroup):
            return super()._add_action(action)

        # Split the destination into a list of keys
        keys = action.dest.split(".")
        group = None

        for i in range(1, len(keys)):
            ns_key = ".".join(keys[:-i])
            for action_group in self._action_groups:
                if (
                    isinstance(action_group, _NamespaceArgumentGroup)
                    and action_group.name == ns_key
                ):
                    group = action_group
                    break
            if group is not None:
                break

        if group is None:
            group = self.add_namespace(keys[0])

        return group._add_action(action)

    def add_namespace(
        self,
        name: str,
        title: str | None = None,
        **kwargs,
    ) -> _NamespaceArgumentGroup:
        """Add a namespace to the parser, which is actually an argument group

        Args:
            name (str): The name of the namespace
                The name should match the first part of the dest of its actions.
            title (str, optional): The title of the namespace.
            **kwargs: The arguments to pass to add_argument_group()

        Returns:
            _NamespaceArgumentGroup: The namespace
        """
        # Check if the namespace already exists
        for action_group in self._action_groups:
            if (
                isinstance(action_group, _NamespaceArgumentGroup)
                and action_group.name == name
            ):
                raise ValueError(f"Namespace '{name}' already exists")

        if title is None:
            title = f"{_('namespace')} '{name}'"

        group = _NamespaceArgumentGroup(self, title, name=name, **kwargs)
        self._action_groups.append(group)
        return group

    def add_argument_group(self, *args, **kwargs) -> _ArgumentGroup:
        """Add an argument group to the parser.

        Modify to handle show.
        """
        group = _ArgumentGroup(self, *args, **kwargs)
        self._action_groups.append(group)
        return group

    def print_help(  # type: ignore[override]
        self,
        plus: bool = False,
        file: IO[str] | None = None,
    ) -> None:
        """Print the help message.

        Modify to handle plus.
        """
        if file is None:
            file = sys.stdout

        self._print_message(self.format_help(plus=plus), file)

    def format_help(self, plus: bool = True) -> str:
        """Format the help message.

        Modify to handle plus.
        """
        formatter = self._get_formatter()

        # usage
        formatter.add_usage(
            self.usage, self._actions, self._mutually_exclusive_groups
        )

        # description
        formatter.add_text(self.description)

        # positionals, optionals and user-defined groups
        for action_group in self._action_groups:
            if not plus and not showable(action_group):
                for action in action_group._group_actions:
                    # hide them in usage as well
                    action.help = SUPPRESS
                continue
            formatter.start_section(action_group.title)
            formatter.add_text(action_group.description)
            formatter.add_arguments(  # type: ignore[call-arg]
                action_group._group_actions,
                plus,
            )
            formatter.end_section()

        # epilog
        formatter.add_text(self.epilog)

        # determine help from format above
        return formatter.format_help()

    def _add_decedents(
        self,
        mutually_exclusive_groups: list[dict],
        groups: list[dict],
        namespaces: list[dict],
        arguments: list[dict],
        commands: list[dict],
    ) -> None:
        """Add the decedents of the parser"""
        # Add the mutually exclusive groups
        for group_args in mutually_exclusive_groups:
            group_arguments = group_args.pop("arguments", [])
            mgroup = self.add_mutually_exclusive_group(**group_args)
            for argument in group_arguments:
                flags = argument.pop("flags", [])
                mgroup.add_argument(*flags, **argument)

        # Add the groups
        for group_args in groups:
            group_arguments = group_args.pop("arguments", [])
            group = self.add_argument_group(**group_args)
            for argument in group_arguments:
                flags = argument.pop("flags", [])
                group.add_argument(*flags, **argument)

        # Add the namespaces
        for namespace in namespaces:
            namespace_arguments = namespace.pop("arguments", [])
            self.add_namespace(**namespace)
            arguments.extend(namespace_arguments)

        # Add the arguments
        for argument in arguments:
            flags = argument.pop("flags", [])
            self.add_argument(*flags, **argument)

        # Add the commands
        for command_args in commands:
            command_mutually_exclusive_groups = command_args.pop(
                "mutually_exclusive_groups",
                [],
            )
            command_groups = command_args.pop("groups", [])
            command_namespaces = command_args.pop("namespaces", [])
            command_arguments = command_args.pop("arguments", [])
            command_commands = command_args.pop("commands", [])
            command = self.add_command(**command_args)
            command._add_decedents(
                command_mutually_exclusive_groups,
                command_groups,
                command_namespaces,
                command_arguments,
                command_commands,
            )

    def _registry_get(
        self,
        registry_name: str,
        value: Any,
        default: Any = None,
    ) -> Any:
        # Allow type to be string
        if (
            registry_name == "type"
            and isinstance(value, str)
            and default == value
        ):
            import builtins

            # "int", "float", "str", "open", etc
            return self._registries[registry_name].get(
                value,
                getattr(builtins, value, None),
            )

        # Not using super() because _ArgumentGroup can also use it
        return APArgumentGroup._registry_get(
            self,
            registry_name,
            value,
            default,
        )

    @classmethod
    def from_configs(cls, *configs: dict | str) -> ArgumentParser:
        """Create an ArgumentParser from a configuration file"""
        config = Config.load(*configs)
        mutually_exclusive_groups = config.pop("mutually_exclusive_groups", [])
        groups = config.pop("groups", [])
        namespaces = config.pop("namespaces", [])
        arguments = config.pop("arguments", [])
        commands = config.pop("commands", [])
        parser = cls(**config)
        parser._add_decedents(
            mutually_exclusive_groups,
            groups,
            namespaces,
            arguments,
            commands,
        )
        return parser


@add_attribute("show", True)
class _ArgumentGroup(APArgumentGroup):
    _registry_get = ArgumentParser._registry_get


@add_attribute("name")
class _NamespaceArgumentGroup(_ArgumentGroup):
    ...



from __future__ import annotations
from typing import TYPE_CHECKING, Any, Sequence
from argparse import (
    _HelpAction,
    _StoreAction,
    _StoreConstAction,
    _AppendAction,
    _AppendConstAction,
    _CountAction,
    # Introduced in python3.9
    # _ExtendAction,
)

from .utils import get_ns_dest, copy_items, add_attribute

if TYPE_CHECKING:
    from argparse import Namespace
    from .parser import ArgumentParser


@add_attribute("show", True)
class StoreAction(_StoreAction):
    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        ns, dest = get_ns_dest(namespace, self.dest)
        setattr(ns, dest, values)


@add_attribute("show", True)
class StoreConstAction(_StoreConstAction):
    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        ns, dest = get_ns_dest(namespace, self.dest)
        setattr(ns, dest, self.const)


class StoreTrueAction(StoreConstAction):
    def __init__(self, *args, **kwargs):
        kwargs["const"] = True
        super().__init__(*args, **kwargs)


class StoreFalseAction(StoreConstAction):
    def __init__(self, *args, **kwargs):
        kwargs["const"] = False
        super().__init__(*args, **kwargs)


@add_attribute("show", True)
class AppendAction(_AppendAction):
    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        ns, dest = get_ns_dest(namespace, self.dest)
        items = getattr(ns, dest, None)
        items = copy_items(items)
        items.append(values)
        setattr(ns, dest, items)


@add_attribute("show", True)
class AppendConstAction(_AppendConstAction):
    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        ns, dest = get_ns_dest(namespace, self.dest)
        items = getattr(ns, dest, None)
        items = copy_items(items)
        items.append(self.const)
        setattr(ns, dest, items)


@add_attribute("show", True)
class CountAction(_CountAction):
    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        ns, dest = get_ns_dest(namespace, self.dest)
        value = getattr(ns, dest, None)
        if value is None:
            value = 0
        setattr(ns, dest, value + 1)


class ExtendAction(AppendAction):
    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        ns, dest = get_ns_dest(namespace, self.dest)
        items = getattr(ns, dest, None)
        items = copy_items(items)
        items.extend(values)
        setattr(ns, dest, items)


class ListAction(AppendAction):
    """Append a list of values to the list of values for a given option"""

    def __init__(self, *args, **kwargs):
        self.received = False
        super().__init__(*args, **kwargs)

    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        ns, dest = get_ns_dest(namespace, self.dest)
        if not self.received:
            items = []
            self.received = True
        else:
            items = getattr(ns, dest, None)
            items = copy_items(items)

        items.append(values)
        setattr(ns, dest, items)


@add_attribute("show", True)
class HelpAction(_HelpAction):
    def __call__(  # type: ignore[override]
        self,
        parser: ArgumentParser,
        namespace: Namespace,
        values: str | Sequence[Any] | None,
        option_string: str | None = None,
    ) -> None:
        parser.print_help(
            plus=(
                (parser.add_help == "+" and "+" in option_string)
                or parser.add_help is True
            )
        )
        parser.exit()



"""Additional types for argx"""
from typing import Any


def py(s: str) -> Any:
    from ast import literal_eval
    return literal_eval(s)


def json(s: str) -> Any:
    import json
    return json.loads(s)


def auto(s: str) -> Any:
    if s in ("True", "TRUE", "true"):
        return True
    if s in ("False", "FALSE", "false"):
        return False

    try:
        return int(s)
    except (TypeError, ValueError):
        pass

    try:
        return float(s)
    except (TypeError, ValueError):
        pass

    import json
    try:
        return json.loads(s)
    except (TypeError, ValueError, json.JSONDecodeError):
        pass

    return s



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['argx']

package_data = \
{'': ['*']}

install_requires = \
['python-simpleconf>=0.5.7,<0.6.0']

setup_kwargs = {
    'name': 'argx',
    'version': '0.0.2',
    'description': 'Super-charged argparse for python',
    'long_description': '# argx\n\n[![pypi][1]][2] [![tag][3]][2] [![codacy quality][4]][5] [![codacy quality][6]][5] ![github action][7] ![pyver][8]\n\nSupercharged argparse for Python\n\n## Installation\n\n```bash\npip install -U argx\n```\n\n## Features\n\n- Option `exit_on_void`: Exit with error if no arguments are provided\n- Subcommand shortcut: Adding subcommands directly, without `parser.add_subparsers()`\n- Namespace arguments: Access arguments like `--foo.bar` as `args.foo.bar`\n- Brief help message for massive arguments: Show only the most important arguments in the help message\n- Defaults from file: Read default values from a configuration file by API or from command line\n- List action: Store a list of values. Different from `append` and `extend`, the initial value is cleared.\n- Addtional types: Some additional types to convert the values of arguments\n- Configuration file to create the parser: Instead of creating the parser by code, you can also create it by a configuration file\n- Backward compatibility: All features are optional. You can use `argx` as a drop-in replacement for `argparse`.\n\n### Option `exit_on_void`\n\nIf all arguments are optional, `argparse` will not raise an error if no arguments are provided. This is not always desirable. `argx` provides the option `exit_on_void` to change this behavior. If `exit_on_void` is set to `True` and no arguments are provided, `argx` will exit with an error (No arguments provided).\n\n```python\nimport argx as argparse\n\nparser = argparse.ArgumentParser(exit_on_void=True)\nparser.add_argument(\'--foo\', action=\'store_true\')\n\nargs = parser.parse_args([])\n# No arguments provided\n# standard argparse produces: Namespace(foo=False)\n```\n\n### Subcommand shortcut\n\n`argparse` requires to create subparsers first and then add the subcommands to the subparsers.\n`argx` allows to add subcommands directly to the main parser.\n\n```python\n# standard argparse\nparser = argparse.ArgumentParser()\nsubparsers = parser.add_subparsers(title=\'subcommands\')\nparser_a = subparsers.add_parser(\'a\')\nparser_b = subparsers.add_parser(\'b\')\n\n# argx\nparser = argparse.ArgumentParser()\nparser.add_command(\'a\')  # or parser.add_subparser(\'a\')\nparser.add_command(\'b\')\n# args = parser.parse_args([\'a\'])\n# Namespace(COMMAND=\'a\')\n```\n\nThe `subparsers` is added automatically with the title `subcommands` and the `dest` is set to `COMMAND`. You can add subcommands to subcommands directly, then the `dest` is set to `COMMAND2`, `COMMAND3`, etc. If you want to change the behavior, you can always fall back to the standard `argparse` way.\n\n### Namespace arguments\n\nThe values of arguments like `--foo.bar` can be accessed as `vars(args)[\'foo.bar\']`. With `argx` you can access them as `args.foo.bar`.\n\nThe arguments `--foo.bar`, `--foo.baz` and `--foo.qux` are automatically grouped in a namespace `foo`.\n\n```python\nimport argx as argparse\n\nparser = argparse.ArgumentParser()\nparser.add_argument(\'--foo.bar\', type=int)\nparser.add_argument(\'--foo.baz\', type=int)\nparser.add_argument(\'--foo.qux\', type=int)\n\nparser.print_help()\n```\n\n```shell\nusage: test.py [-h] [--foo.bar BAR] [--foo.baz BAZ] [--foo.qux QUX]\n\noptional arguments:\n  -h, --help            show this help message and exit\n\nnamespace \'foo\':\n  --foo.bar BAR\n  --foo.baz BAZ\n  --foo.qux QUX\n```\n\nYou can modify the namespace by adding the namespace manually before adding the arguments.\n\n```python\nimport argx as argparse\n\nparser = argparse.ArgumentParser()\nparser.add_namespace(\'foo\', title="Foo\'s options")\nparser.add_argument(\'--foo.bar\', type=int)\nparser.add_argument(\'--foo.baz\', type=int)\nparser.add_argument(\'--foo.qux\', type=int)\n\nparser.print_help()\n```\n\n```shell\nusage: test.py [-h] [--foo.bar BAR] [--foo.baz BAZ] [--foo.qux QUX]\n\noptional arguments:\n  -h, --help            show this help message and exit\n\nFoo\'s options:\n    --foo.bar BAR\n    --foo.baz BAZ\n    --foo.qux QUX\n```\n\n### Brief help message for massive arguments\n\nIf you have a lot of arguments, the help message can be very long. `argx` allows to show only the most important arguments in the help message.\n\n```python\nimport argx as argparse\n\n# Advanced help options to show the brief help message or the full help message\nparser = argparse.ArgumentParser(add_help=\'+\')\nparser.add_argument(\'--foo\', type=int)\nparser.add_argument(\'--bar\', type=int, show=False)\nparser.parse_args([\'--help\'])\n```\n\n```shell\nusage: test.py [-h] [--foo FOO]\n\noptional arguments:\n  -h, --help, -h+, --help+\n                        show help message (with + to show more options) and exit\n  --foo FOO\n```\n\nWith `parser.parse_args([\'--help+\'])` you can show the full help message.\n\n```shell\nusage: test.py [-h] [--foo FOO] [--bar BAR]\n\noptional arguments:\n  -h, --help, -h+, --help+\n                        show help message (with + to show more options) and exit\n  --foo FOO\n  --bar BAR\n```\n\nYou can also set `show=False` for argument groups.\n\n### Defaults from files\n\nWith standard `argparse`, when `fromfile_prefix_chars` is set, the arguments can be read from a file. The file can be specified with `@filename`. The arguments in the file are separated by newlines by default.\n\nWith `argx`, Other than a text file to provide command line arguments, you can also provide other types of configuration files. The extension of the file can be `.json`, `.yaml`, `.ini`, `.env` or `.toml`.\n\n```python\nimport argx as argparse\n\nparser = argparse.ArgumentParser()\nparser.add_argument(\'--foo\', type=int)\nparser.add_argument(\'--bar\', type=int)\nparser.add_argument(\'--baz\', type=int)\n\n# config.json\n# { "foo": 1, "bar": 2, "baz": 3 }\nargs = parser.parse_args([\'@config.json\'])\n# Namespace(foo=1, bar=2, baz=3)\n```\n\nYou can also use `set_defaults_from_configs` method:\n\n```python\nparser.set_defaults_from_configs(\'config.json\')\n```\n\n### List action\n\nThe `list` action is similar to `append` and `extend`, but the initial value is cleared.\n\nThis is useful when you want to accept a new list of values from the command line, instead of appending to the existing list or default.\n\n```python\nimport argx as argparse\n\nparser = argparse.ArgumentParser()\nparser.add_argument(\'--foo\', action=\'list\', default=[1, 2, 3], type=int)\nparser.add_argument(\'--bar\', action=\'append\', default=[1, 2, 3], type=int)\n\nargs = parser.parse_args(\'--foo 4 --foo 5 --bar 4 --bar 5\'.split())\n# Namespace(foo=[4, 5], bar=[1, 2, 3, 4, 5])\n```\n\n### Additional types\n\n`parser.add_argument()` accepts `type` as a function to convert the argument value. It has to be a callable that accepts a single string argument and returns the converted value. While `argx` supports string for `type` so it can be configured in the configuration file. Builtin functions and types can also be specified by its name.\n\nWe also have additional types:\n\n```python\nimport argx as argparse\n\nparser = argparse.ArgumentParser()\nparser.add_argument(\'--foo\', type=\'py\')\nparser.add_argument(\'--bar\', type=\'json\')\nparser.add_argument(\'--baz\', type=\'path\')\nparser.add_argument(\'--qux\', type=\'auto\')\n\nargs = parser.parse_args(\n    \'--foo 1 --bar {"a":1} --baz path/to/file --qux true\'.split()\n)\n# Namespace(foo=1, bar={\'a\': 1}, baz=PosixPath(\'path/to/file\'), qux=True)\n```\n\n- `py`: Python expression. The string is evaluated by `ast.literal_eval`.\n- `json`: JSON string. The string is loaded by `json.loads`.\n- `path`: Path string. The string is converted by `pathlib.Path`.\n- `auto`: Automatic type conversion.\n  - `True` if the string is `True/TRUE/true`\n  - `False` if the string is `False/FALSE/false`\n  - An integer if the string can be converted to an integer\n  - A float if the string can be converted to a float\n  - A dict if the string is a JSON string\n  - The string itself otherwise\n\n### Configuration file to create the parser\n\nYou can create the parser from a configuration file.\n\n```python\nimport argx as argparse\n\n# config.json\n# {\n#   "prog": "myprog",\n#   "arguments": [ {"flags": ["-a", "--abc"], "help": "Optiona a help"} ]\n# }\nparser = argparse.ArgumentParser.from_configs(\'config.json\')\nparser.print_help()\n```\n\n```shell\nusage: myprog [-h] [-a ABC]\n\noptional arguments:\n  -h, --help            show help message and exit\n  -a ABC, --abc ABC     Optiona a help\n```\n\n### Backward compatibility\n\nAll features are optional. You can use `argx` as a drop-in replacement for `argparse`.\n\n`argx` supports python `3.7+`. Some of the later-introduced features are also supported in python 3.7. For example, `extend` action is added in python 3.8, `argx` supports in python 3.7.\n\n[1]: https://img.shields.io/pypi/v/argx.svg?style=flat-square\n[2]: https://pypi.org/project/argx/\n[3]: https://img.shields.io/github/tag/pwwang/argx.svg?style=flat-square\n[4]: https://img.shields.io/codacy/grade/c5eaafcde482437b901b1acd2b70420e.svg?style=flat-square\n[5]: https://app.codacy.com/gh/pwwang/argx/dashboard\n[6]: https://img.shields.io/codacy/coverage/c5eaafcde482437b901b1acd2b70420e.svg?style=flat-square\n[7]: https://img.shields.io/github/actions/workflow/status/pwwang/argx/build.yml?style=flat-square\n[8]: https://img.shields.io/pypi/pyversions/argx.svg?style=flat-square\n',
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import Callable, Iterable, Tuple
from argparse import SUPPRESS, Action, HelpFormatter, _SubParsersAction

from .utils import showable


class ChargedHelpFormatter(HelpFormatter):
    """Help formatter for pyparam"""

    def _format_action(self, action: Action) -> str:
        """Skip header for subparsers"""
        if isinstance(action, _SubParsersAction):
            parts = []
            for subaction in self._iter_indented_subactions(action):
                parts.append(super()._format_action(subaction))
            return self._join_parts(parts)

        return super()._format_action(action)

    def _metavar_formatter(
        self,
        action: Action,
        default_metavar: str,
    ) -> Callable[[int], Tuple[str, ...]]:
        """Format metavar in case there are namespace in it"""
        fmt = super()._metavar_formatter(action, default_metavar)

        def format(tuple_size):
            result = fmt(tuple_size)
            return tuple(
                r.split(".")[-1] if isinstance(r, str) else r for r in result
            )

        return format

    def add_arguments(  # type: ignore[override]
        self,
        actions: Iterable[Action],
        plus: bool,
    ) -> None:
        """Add arguments to the formatter

        Modify to handle plus.
        """
        for action in actions:
            if not plus and not showable(action):
                action.help = SUPPRESS
            self.add_argument(action)
